@@ -1,4 +1,4 @@
-#! /usr/bin/python3
+#!/usr/bin/python3
 # -*- python -*-
 # -*- coding: utf-8 -*-
 #
@@ -18,37 +18,46 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 
-import os, time
-from functools import reduce
-from six.moves import range
-from .utilist import bitmasklist
+import os
 import platform
 import re
+import time
+from functools import reduce
+from six.moves import range
+from procfs.utilist import bitmasklist
 
 VERSION = "0.5"
 
+
 def is_s390():
+    """ Return True if running on s390 or s390x """
     machine = platform.machine()
-    if re.search('s390', machine):
-        return True
-    else:
-        return False
+    return bool(re.search('s390', machine))
+
 
 def process_cmdline(pid_info):
     """
-    Returns the process command line, if available in the given `process' class, if
-    not available, falls back to using the comm (short process name) in its pidstat key.
+    Returns the process command line, if available in the given `process' class,
+    if not available, falls back to using the comm (short process name) in its
+    pidstat key.
     """
     if pid_info["cmdline"]:
         return reduce(lambda a, b: a + " %s" % b, pid_info["cmdline"]).strip()
 
-    return pid_info["stat"]["comm"]
+    try:
+        """ If a pid disappears before we query it, return None """
+        return pid_info["stat"]["comm"]
+    except:
+        return None
+
 
 class pidstat:
-    """Provides a dictionary to access the fields in the per process /proc/PID/stat
-       files.
+    """
+    Provides a dictionary to access the fields in the
+    per process /proc/PID/stat files.
 
-       One can obtain the available fields asking for the keys of the dictionary, e.g.:
+    One can obtain the available fields by asking for the keys of the
+    dictionary, e.g.:
 
         >>> p = procfs.pidstat(1)
         >>> print p.keys()
@@ -74,58 +83,66 @@ class pidstat:
     # Entries with the same value, the one with a comment after it is the
     # more recent, having replaced the other name in v4.1-rc kernel times.
 
-    PF_ALIGNWARN     = 0x00000001
-    PF_STARTING     = 0x00000002
-    PF_EXITING     = 0x00000004
-    PF_EXITPIDONE     = 0x00000008
-    PF_VCPU         = 0x00000010
-    PF_WQ_WORKER     = 0x00000020 # /* I'm a workqueue worker */
-    PF_FORKNOEXEC     = 0x00000040
-    PF_MCE_PROCESS     = 0x00000080 # /* process policy on mce errors */
-    PF_SUPERPRIV     = 0x00000100
-    PF_DUMPCORE     = 0x00000200
-    PF_SIGNALED     = 0x00000400
-    PF_MEMALLOC     = 0x00000800
-    PF_NPROC_EXCEEDED= 0x00001000 # /* set_user noticed that RLIMIT_NPROC was exceeded */
-    PF_FLUSHER     = 0x00001000
-    PF_USED_MATH     = 0x00002000
-    PF_USED_ASYNC     = 0x00004000 # /* used async_schedule*(), used by module init */
-    PF_NOFREEZE     = 0x00008000
-    PF_FROZEN     = 0x00010000
-    PF_FSTRANS     = 0x00020000
-    PF_KSWAPD     = 0x00040000
-    PF_MEMALLOC_NOIO = 0x00080000 # /* Allocating memory without IO involved */
-    PF_SWAPOFF     = 0x00080000
+    PF_ALIGNWARN = 0x00000001
+    PF_STARTING = 0x00000002
+    PF_EXITING = 0x00000004
+    PF_EXITPIDONE = 0x00000008
+    PF_VCPU = 0x00000010
+    PF_WQ_WORKER = 0x00000020  # /* I'm a workqueue worker */
+    PF_FORKNOEXEC = 0x00000040
+    PF_MCE_PROCESS = 0x00000080  # /* process policy on mce errors */
+    PF_SUPERPRIV = 0x00000100
+    PF_DUMPCORE = 0x00000200
+    PF_SIGNALED = 0x00000400
+    PF_MEMALLOC = 0x00000800
+    # /* set_user noticed that RLIMIT_NPROC was exceeded */
+    PF_NPROC_EXCEEDED = 0x00001000
+    PF_FLUSHER = 0x00001000
+    PF_USED_MATH = 0x00002000
+    PF_USED_ASYNC = 0x00004000  # /* used async_schedule*(), used by module init */
+    PF_NOFREEZE = 0x00008000
+    PF_FROZEN = 0x00010000
+    PF_FSTRANS = 0x00020000
+    PF_KSWAPD = 0x00040000
+    PF_MEMALLOC_NOIO = 0x00080000  # /* Allocating memory without IO involved */
+    PF_SWAPOFF = 0x00080000
     PF_LESS_THROTTLE = 0x00100000
-    PF_KTHREAD     = 0x00200000
-    PF_RANDOMIZE     = 0x00400000
-    PF_SWAPWRITE     = 0x00800000
-    PF_SPREAD_PAGE     = 0x01000000
-    PF_SPREAD_SLAB     = 0x02000000
-    PF_THREAD_BOUND     = 0x04000000
-    PF_NO_SETAFFINITY = 0x04000000 # /* Userland is not allowed to meddle with cpus_allowed */
-    PF_MCE_EARLY     = 0x08000000 # /* Early kill for mce process policy */
-    PF_MEMPOLICY     = 0x10000000
-    PF_MUTEX_TESTER     = 0x20000000
-    PF_FREEZER_SKIP     = 0x40000000
+    PF_KTHREAD = 0x00200000
+    PF_RANDOMIZE = 0x00400000
+    PF_SWAPWRITE = 0x00800000
+    PF_SPREAD_PAGE = 0x01000000
+    PF_SPREAD_SLAB = 0x02000000
+    PF_THREAD_BOUND = 0x04000000
+    # /* Userland is not allowed to meddle with cpus_allowed */
+    PF_NO_SETAFFINITY = 0x04000000
+    PF_MCE_EARLY = 0x08000000  # /* Early kill for mce process policy */
+    PF_MEMPOLICY = 0x10000000
+    PF_MUTEX_TESTER = 0x20000000
+    PF_FREEZER_SKIP = 0x40000000
     PF_FREEZER_NOSIG = 0x80000000
-    PF_SUSPEND_TASK     = 0x80000000 # /* this thread called freeze_processes and should not be frozen */
+    # /* this thread called freeze_processes and should not be frozen */
+    PF_SUSPEND_TASK = 0x80000000
 
     proc_stat_fields = ["pid", "comm", "state", "ppid", "pgrp", "session",
-                 "tty_nr", "tpgid", "flags", "minflt", "cminflt",
-                 "majflt", "cmajflt", "utime", "stime", "cutime",
-                 "cstime", "priority", "nice", "num_threads",
-                 "itrealvalue", "starttime", "vsize", "rss",
-                 "rlim", "startcode", "endcode", "startstack",
-                 "kstkesp", "kstkeip", "signal", "blocked",
-                 "sigignore", "sigcatch", "wchan", "nswap",
-                 "cnswap", "exit_signal", "processor",
-                 "rt_priority", "policy",
-                 "delayacct_blkio_ticks", "environ"]
+                        "tty_nr", "tpgid", "flags", "minflt", "cminflt",
+                        "majflt", "cmajflt", "utime", "stime", "cutime",
+                        "cstime", "priority", "nice", "num_threads",
+                        "itrealvalue", "starttime", "vsize", "rss",
+                        "rlim", "startcode", "endcode", "startstack",
+                        "kstkesp", "kstkeip", "signal", "blocked",
+                        "sigignore", "sigcatch", "wchan", "nswap",
+                        "cnswap", "exit_signal", "processor",
+                        "rt_priority", "policy",
+                        "delayacct_blkio_ticks", "environ"]
 
     def __init__(self, pid, basedir="/proc"):
         self.pid = pid
-        self.load(basedir)
+        try:
+            self.load(basedir)
+        except FileNotFoundError:
+            # The file representing the pid has disappeared
+            #  propagate the error to the user to handle
+            raise
 
     def __getitem__(self, fieldname):
         return self.fields[fieldname]
@@ -146,7 +163,11 @@ class pidstat:
         return fieldname in self.fields
 
     def load(self, basedir="/proc"):
-        f = open("%s/%d/stat" % (basedir, self.pid))
+        try:
+            f = open("%s/%d/stat" % (basedir, self.pid))
+        except FileNotFoundError:
+            # The pid has disappeared, propagate the error
+            raise
         fields = f.readline().strip().split(') ')
         f.close()
         fields = fields[0].split(' (') + fields[1].split()
@@ -168,8 +189,7 @@ class pidstat:
         Returns true if this process has a fixed smp affinity mask,
                 not allowing it to be moved to a different set of CPUs.
         """
-        return self.fields["flags"] & self.PF_THREAD_BOUND and \
-            True or False
+        return bool(self.fields["flags"] & self.PF_THREAD_BOUND)
 
     def process_flags(self):
         """
@@ -179,33 +199,34 @@ class pidstat:
 
         As of v4.2-rc7 these include (from include/linux/sched.h comments):
 
-            PF_EXITING       Getting shut down
-            PF_EXITPIDONE       Pi exit done on shut down
+            PF_EXITING        Getting shut down
+            PF_EXITPIDONE     Pi exit done on shut down
             PF_VCPU           I'm a virtual CPU
-            PF_WQ_WORKER       I'm a workqueue worker
-            PF_FORKNOEXEC       Forked but didn't exec
-            PF_MCE_PROCESS       Process policy on mce errors
-            PF_SUPERPRIV       Used super-user privileges
+            PF_WQ_WORKER      I'm a workqueue worker
+            PF_FORKNOEXEC     Forked but didn't exec
+            PF_MCE_PROCESS    Process policy on mce errors
+            PF_SUPERPRIV      Used super-user privileges
             PF_DUMPCORE       Dumped core
             PF_SIGNALED       Killed by a signal
             PF_MEMALLOC       Allocating memory
-            PF_NPROC_EXCEEDED  Set_user noticed that RLIMIT_NPROC was exceeded
-            PF_USED_MATH       If unset the fpu must be initialized before use
-            PF_USED_ASYNC       Used async_schedule*(), used by module init
+            PF_NPROC_EXCEEDED Set_user noticed that RLIMIT_NPROC was exceeded
+            PF_USED_MATH      If unset the fpu must be initialized before use
+            PF_USED_ASYNC     Used async_schedule*(), used by module init
             PF_NOFREEZE       This thread should not be frozen
-            PF_FROZEN       Frozen for system suspend
-            PF_FSTRANS       Inside a filesystem transaction
-            PF_KSWAPD       I am kswapd
+            PF_FROZEN          Frozen for system suspend
+            PF_FSTRANS         Inside a filesystem transaction
+            PF_KSWAPD          I am kswapd
             PF_MEMALLOC_NOIO   Allocating memory without IO involved
             PF_LESS_THROTTLE   Throttle me less: I clean memory
-            PF_KTHREAD       I am a kernel thread
+            PF_KTHREAD         I am a kernel thread
             PF_RANDOMIZE       Randomize virtual address space
             PF_SWAPWRITE       Allowed to write to swap
             PF_NO_SETAFFINITY  Userland is not allowed to meddle with cpus_allowed
             PF_MCE_EARLY       Early kill for mce process policy
-            PF_MUTEX_TESTER       Thread belongs to the rt mutex tester
-            PF_FREEZER_SKIP       Freezer should not count it as freezable
-            PF_SUSPEND_TASK       This thread called freeze_processes and should not be frozen
+            PF_MUTEX_TESTER    Thread belongs to the rt mutex tester
+            PF_FREEZER_SKIP    Freezer should not count it as freezable
+            PF_SUSPEND_TASK    This thread called freeze_processes and
+                               should not be frozen
 
         """
         sflags = []
@@ -218,29 +239,34 @@ class pidstat:
 
         return sflags
 
+
 def cannot_set_affinity(self, pid):
     PF_NO_SETAFFINITY = 0x04000000
     try:
-        return int(self.processes[pid]["stat"]["flags"]) & \
-          PF_NO_SETAFFINITY and True or False
+        return bool(int(self.processes[pid]["stat"]["flags"]) &
+                    PF_NO_SETAFFINITY)
     except:
         return True
 
+
 def cannot_set_thread_affinity(self, pid, tid):
     PF_NO_SETAFFINITY = 0x04000000
     try:
-        return int(self.processes[pid].threads[tid]["stat"]["flags"]) & \
-          PF_NO_SETAFFINITY and True or False
+        return bool(int(self.processes[pid].threads[tid]["stat"]["flags"]) &
+                    PF_NO_SETAFFINITY)
     except:
         return True
 
+
 class pidstatus:
     """
-    Provides a dictionary to access the fields in the per process /proc/PID/status
-    files. This provides additional information about processes and threads to what
-    can be obtained with the procfs.pidstat() class.
+    Provides a dictionary to access the fields
+    in the per process /proc/PID/status files.
+    This provides additional information about processes and threads to
+    what can be obtained with the procfs.pidstat() class.
 
-    One can obtain the available fields asking for the keys of the dictionary, e.g.:
+    One can obtain the available fields by asking for the keys of the
+    dictionary, e.g.:
 
         >>> import procfs
         >>> p = procfs.pidstatus(1)
@@ -295,19 +321,19 @@ class pidstatus:
         return fieldname in self.fields
 
     def load(self, basedir="/proc"):
-        f = open("%s/%d/status" % (basedir, self.pid))
         self.fields = {}
-        for line in f.readlines():
-            fields = line.split(":")
-            if len(fields) != 2:
-                continue
-            name = fields[0]
-            value = fields[1].strip()
-            try:
-                self.fields[name] = int(value)
-            except:
-                self.fields[name] = value
-        f.close()
+        with open("%s/%d/status" % (basedir, self.pid)) as f:
+            for line in f.readlines():
+                fields = line.split(":")
+                if len(fields) != 2:
+                    continue
+                name = fields[0]
+                value = fields[1].strip()
+                try:
+                    self.fields[name] = int(value)
+                except:
+                    self.fields[name] = value
+
 
 class process:
     """
@@ -329,7 +355,11 @@ class process:
                 else:
                     sclass = pidstatus
 
-                setattr(self, attr, sclass(self.pid, self.basedir))
+                try:
+                    setattr(self, attr, sclass(self.pid, self.basedir))
+                except FileNotFoundError:
+                    # The pid has disappeared, progate the error
+                    raise
             elif attr == "cmdline":
                 self.load_cmdline()
             elif attr == "threads":
@@ -348,9 +378,15 @@ class process:
         return hasattr(self, attr)
 
     def load_cmdline(self):
-        f = open("/proc/%d/cmdline" % self.pid)
-        self.cmdline = f.readline().strip().split('\0')[:-1]
-        f.close()
+        try:
+            with open("/proc/%d/cmdline" % self.pid) as f:
+                self.cmdline = f.readline().strip().split('\0')[:-1]
+        except FileNotFoundError:
+            """ This can happen when a pid disappears """
+            self.cmdline = None
+        except UnicodeDecodeError:
+            """ TODO - this shouldn't happen, needs to be investigated """
+            self.cmdline = None
 
     def load_threads(self):
         self.threads = pidstats("/proc/%d/task/" % self.pid)
@@ -358,14 +394,13 @@ class process:
         del self.threads[self.pid]
 
     def load_cgroups(self):
-        f = open("/proc/%d/cgroup" % self.pid)
         self.cgroups = ""
-        for line in reversed(f.readlines()):
-            if len(self.cgroups):
-                self.cgroups = self.cgroups + "," + line[:-1]
-            else:
-                self.cgroups = line[:-1]
-        f.close()
+        with open("/proc/%d/cgroup" % self.pid) as f:
+            for line in reversed(f.readlines()):
+                if len(self.cgroups) != 0:
+                    self.cgroups = self.cgroups + "," + line[:-1]
+                else:
+                    self.cgroups = line[:-1]
 
     def load_environ(self):
         """
@@ -394,12 +429,12 @@ class process:
         >>>
         """
         self.environ = {}
-        f = open("/proc/%d/environ" % self.pid)
-        for x in f.readline().split('\0'):
-            if len(x) > 0:
-                y = x.split('=')
-                self.environ[y[0]] = y[1]
-        f.close()
+        with open("/proc/%d/environ" % self.pid) as f:
+            for x in f.readline().split('\0'):
+                if len(x) > 0:
+                    y = x.split('=')
+                    self.environ[y[0]] = y[1]
+
 
 class pidstats:
     """
@@ -409,6 +444,7 @@ class pidstats:
     The entries can be accessed as a dictionary, keyed by pid. Also there are
     methods to find processes that match a given COMM or regular expression.
     """
+
     def __init__(self, basedir="/proc"):
         self.basedir = basedir
         self.processes = {}
@@ -441,18 +477,18 @@ class pidstats:
 
     def reload(self):
         """
-        This operation will throw away the current dictionary contents, if any, and
-        read all the pid files from /proc/, instantiating a 'process' instance for
-        each of them.
+        This operation will throw away the current dictionary contents,
+        if any, and read all the pid files from /proc/, instantiating a
+        'process' instance for each of them.
 
-        This is a high overhead operation, and should be avoided if the perf python
-        binding can be used to detect when new threads appear and existing ones
-        terminate.
+        This is a high overhead operation, and should be avoided if the
+        perf python binding can be used to detect when new threads appear
+        and existing ones terminate.
 
         In RHEL it is found in the python-perf rpm package.
 
-        More information about the perf facilities can be found in the 'perf_event_open'
-        man page.
+        More information about the perf facilities can be found in the
+        'perf_event_open' man page.
         """
         del self.processes
         self.processes = {}
@@ -568,6 +604,7 @@ class pidstats:
         """
         return self.processes[pid]["stat"].is_bound_to_cpu()
 
+
 class interrupts:
     """
     Information about IRQs in the system. A dictionary keyed by IRQ number
@@ -592,6 +629,7 @@ class interrupts:
     {'affinity': [0, 1, 2, 3], 'type': 'PCI-MSI', 'cpu': [3495, 0, 81, 0], 'users': ['thunderbolt']}
     >>>
     """
+
     def __init__(self):
         self.interrupts = {}
         self.reload()
@@ -617,24 +655,21 @@ class interrupts:
     def reload(self):
         del self.interrupts
         self.interrupts = {}
-        f = open("/proc/interrupts")
-
-        for line in f.readlines():
-            line = line.strip()
-            fields = line.split()
-            if fields[0][:3] == "CPU":
-                self.nr_cpus = len(fields)
-                continue
-            irq = fields[0].strip(":")
-            self.interrupts[irq] = {}
-            self.interrupts[irq] = self.parse_entry(fields[1:], line)
-            try:
-                nirq = int(irq)
-            except:
-                continue
-            self.interrupts[irq]["affinity"] = self.parse_affinity(nirq)
-
-        f.close()
+        with open("/proc/interrupts") as f:
+            for line in f.readlines():
+                line = line.strip()
+                fields = line.split()
+                if fields[0][:3] == "CPU":
+                    self.nr_cpus = len(fields)
+                    continue
+                irq = fields[0].strip(":")
+                self.interrupts[irq] = {}
+                self.interrupts[irq] = self.parse_entry(fields[1:], line)
+                try:
+                    nirq = int(irq)
+                except:
+                    continue
+                self.interrupts[irq]["affinity"] = self.parse_affinity(nirq)
 
     def parse_entry(self, fields, line):
         dict = {}
@@ -647,16 +682,16 @@ class interrupts:
                 dict["type"] = fields[self.nr_cpus]
                 # look if there are users (interrupts 3 and 4 haven't)
                 if nr_fields > self.nr_cpus + 1:
-                    dict["users"] = [a.strip() for a in fields[nr_fields - 1].split(',')]
+                    dict["users"] = [a.strip()
+                                     for a in fields[nr_fields - 1].split(',')]
                 else:
                     dict["users"] = []
         return dict
 
     def parse_affinity(self, irq):
         try:
-            f = open("/proc/irq/%s/smp_affinity" % irq)
-            line = f.readline()
-            f.close()
+            with open("/proc/irq/%s/smp_affinity" % irq) as f:
+                line = f.readline()
             return bitmasklist(line, self.nr_cpus)
         except IOError:
             return [0, ]
@@ -709,15 +744,16 @@ class interrupts:
                     break
         return irqs
 
+
 class cmdline:
     """
     Parses the kernel command line (/proc/cmdline), turning it into a dictionary."
 
-    Useful to figure out if some kernel boolean knob has been turned on, as well
-    as to find the value associated to other kernel knobs.
+    Useful to figure out if some kernel boolean knob has been turned on,
+    as well as to find the value associated to other kernel knobs.
 
-    It can also be used to find out about parameters passed to the init process,
-    such as 'BOOT_IMAGE', etc.
+    It can also be used to find out about parameters passed to the
+    init process, such as 'BOOT_IMAGE', etc.
 
     E.g.:
     >>> import procfs
@@ -734,15 +770,13 @@ class cmdline:
         self.parse()
 
     def parse(self):
-        f = open("/proc/cmdline")
-        for option in f.readline().strip().split():
-            fields = option.split("=")
-            if len(fields) == 1:
-                self.options[fields[0]] = True
-            else:
-                self.options[fields[0]] = fields[1]
-
-        f.close()
+        with open("/proc/cmdline") as f:
+            for option in f.readline().strip().split():
+                fields = option.split("=")
+                if len(fields) == 1:
+                    self.options[fields[0]] = True
+                else:
+                    self.options[fields[0]] = fields[1]
 
     def __getitem__(self, key):
         return self.options[key]
@@ -756,14 +790,15 @@ class cmdline:
     def items(self):
         return self.options
 
+
 class cpuinfo:
     """
     Dictionary with information about CPUs in the system.
 
     Please refer to 'man procfs(5)' for further information about the
-        '/proc/cpuinfo' file, that is the source of the information provided by this
-        class. The 'man lscpu(1)' also has information about a program that uses
-    the '/proc/cpuinfo' file.
+    '/proc/cpuinfo' file, that is the source of the information provided
+    by this class. The 'man lscpu(1)' also has information about a program that
+    uses the '/proc/cpuinfo' file.
 
     Using this class one can obtain the number of CPUs in a system:
 
@@ -772,14 +807,14 @@ class cpuinfo:
           4
 
     It is also possible to figure out aspects of the CPU topology, such as
-    how many CPU physical sockets exists, i.e. groups of CPUs sharing components
-    such as CPU memory caches:
+    how many CPU physical sockets exists, i.e. groups of CPUs sharing
+    components such as CPU memory caches:
 
       >>> print len(cpus.sockets)
       1
 
-    Additionally dictionary with information common to all CPUs in the system is
-    available:
+    Additionally dictionary with information common to all CPUs in the system
+    is available:
 
       >>> print cpus["model name"]
           Intel(R) Core(TM) i7-3667U CPU @ 2.00GHz
@@ -787,6 +822,7 @@ class cpuinfo:
           4096 KB
           >>>
     """
+
     def __init__(self, filename="/proc/cpuinfo"):
         self.tags = {}
         self.nr_cpus = 0
@@ -806,42 +842,45 @@ class cpuinfo:
         return self.tags
 
     def parse(self, filename):
-        f = open(filename)
-        for line in f.readlines():
-            line = line.strip()
-            if not line:
-                continue
-            fields = line.split(":")
-            tagname = fields[0].strip().lower()
-            if tagname == "processor":
-                self.nr_cpus += 1
-                continue
-            elif is_s390() and tagname == "cpu number":
-                self.nr_cpus += 1
-                continue
-            elif tagname == "core id":
-                continue
-            self.tags[tagname] = fields[1].strip()
-            if tagname == "physical id":
-                socket_id = self.tags[tagname]
-                if socket_id not in self.sockets:
-                    self.sockets.append(socket_id)
-
-        f.close()
+        with open(filename) as f:
+            for line in f.readlines():
+                line = line.strip()
+                if not line:
+                    continue
+                fields = line.split(":")
+                tagname = fields[0].strip().lower()
+                if tagname == "processor":
+                    self.nr_cpus += 1
+                    continue
+                if is_s390() and tagname == "cpu number":
+                    self.nr_cpus += 1
+                    continue
+                if tagname == "core id":
+                    continue
+                self.tags[tagname] = fields[1].strip()
+                if tagname == "physical id":
+                    socket_id = self.tags[tagname]
+                    if socket_id not in self.sockets:
+                        self.sockets.append(socket_id)
         self.nr_sockets = self.sockets and len(self.sockets) or \
-                  (self.nr_cpus / ("siblings" in self.tags and int(self.tags["siblings"]) or 1))
-        self.nr_cores = ("cpu cores" in self.tags and int(self.tags["cpu cores"]) or 1) * self.nr_sockets
+            (self.nr_cpus /
+             ("siblings" in self.tags and int(self.tags["siblings"]) or 1))
+        self.nr_cores = ("cpu cores" in self.tags and int(
+            self.tags["cpu cores"]) or 1) * self.nr_sockets
+
 
 class smaps_lib:
     """
     Representation of an mmap in place for a process. Can be used to figure
     out which processes have an library mapped, etc.
 
-    The 'perm' member can be used to figure out executable mmaps, i.e. libraries.
+    The 'perm' member can be used to figure out executable mmaps,
+    i.e. libraries.
 
     The 'vm_start' and 'vm_end' in turn can be used when trying to resolve
     processor instruction pointer addresses to a symbol name in a library.
     """
+
     def __init__(self, lines):
         fields = lines[0].split()
         self.vm_start, self.vm_end = [int(a, 16) for a in fields[0].split("-")]
@@ -901,6 +940,7 @@ class smaps:
           /usr/lib64/libpthread-2.20.so: r-xp
       ...
     """
+
     def __init__(self, pid):
         self.pid = pid
         self.entries = []
@@ -932,13 +972,12 @@ class smaps:
         return self.entries[index]
 
     def reload(self):
-        f = open("/proc/%d/smaps" % self.pid)
         line = None
-        while True:
-            line = self.parse_entry(f, line)
-            if not line:
-                break
-        f.close()
+        with("/proc/%d/smaps" % self.pid) as f:
+            while True:
+                line = self.parse_entry(f, line)
+                if not line:
+                    break
         self.nr_entries = len(self.entries)
 
     def find_by_name_fragment(self, fragment):
@@ -946,10 +985,11 @@ class smaps:
         for i in range(self.nr_entries):
             if self.entries[i].name and \
                self.entries[i].name.find(fragment) >= 0:
-                   result.append(self.entries[i])
+                result.append(self.entries[i])
 
         return result
 
+
 class cpustat:
     """
     CPU statistics, obtained from a line in the '/proc/stat' file, Please
@@ -973,13 +1013,15 @@ class cpustat:
 
     def __repr__(self):
         s = "< user: %s, nice: %s, system: %s, idle: %s, iowait: %s, irq: %s, softirq: %s" % \
-            (self.user, self.nice, self.system, self.idle, self.iowait, self.irq, self.softirq)
+            (self.user, self.nice, self.system, self.idle,
+             self.iowait, self.irq, self.softirq)
         if hasattr(self, 'steal'):
             s += ", steal: %d" % self.steal
         if hasattr(self, 'guest'):
             s += ", guest: %d" % self.guest
         return s + ">"
 
+
 class cpusstats:
     """
     Dictionary with information about CPUs in the system. First entry in the
@@ -988,6 +1030,7 @@ class cpusstats:
     about the '/proc/stat' file, that is the source of the information provided
     by this class.
     """
+
     def __init__(self, filename="/proc/stat"):
         self.entries = {}
         self.time = None
@@ -1016,18 +1059,17 @@ class cpusstats:
     def reload(self):
         last_entries = self.entries
         self.entries = {}
-        f = open(self.filename)
-        for line in f.readlines():
-            fields = line.strip().split()
-            if fields[0][:3].lower() != "cpu":
-                continue
-            c = cpustat(fields)
-            if c.name == "cpu":
-                idx = 0
-            else:
-                idx = int(c.name[3:]) + 1
-            self.entries[idx] = c
-        f.close()
+        with open(self.filename) as f:
+            for line in f.readlines():
+                fields = line.strip().split()
+                if fields[0][:3].lower() != "cpu":
+                    continue
+                c = cpustat(fields)
+                if c.name == "cpu":
+                    idx = 0
+                else:
+                    idx = int(c.name[3:]) + 1
+                self.entries[idx] = c
         last_time = self.time
         self.time = time.time()
         if last_entries:
@@ -1043,8 +1085,8 @@ class cpusstats:
                     (curr.nice - prev.nice) + \
                     (curr.system - prev.system)
                 curr.usage = (delta / interval_hz) * 100
-                if curr.usage > 100:
-                    curr.usage = 100
+                curr.usage = min(curr.usage, 100)
+
 
 if __name__ == '__main__':
     import sys
